#
# Init script for the Coldfire 5206 processor.
# Adapted from a similar gdb.ini script

#	Copyright (C) 2003 Embedded CL S.A.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from pybdm import *

# RAMBAR
# BA[31:13] xxxx WP 00 CI SC SD UC UD V

# MBAR (0xC0F movec)
# BA[31:10] 00000 SC SD UC UD V

#
#  Setup CSAR0 for the FLASH ROM.
#

# Flash: $00000000 - $001fffff (2^21-1, 2MB)
# CPLD:  $00300000 - $00300003 (mod4 $0030ffff)

# CSCR: 0 0 WS3 WS2 WS1 WS0 BRST AA PS1 PS0 EMAA ASET WRAH RDAH WR RD

# Flash: BRST=0 AA=1 PS=11(16b) EMAA=0 ASET=1(CS delay) AH=1(a&d hold)
# %00 1111 0 1 11 0 1 11 11 = 0x3ddf
# %00 0010 0 1 11 0 0 00 11 = 0x09c3 2 wait states (25,50,75 ns)
# %00 0011 0 1 11 0 0 00 11 = 0x0dc3
# %00 0111 0 1 10 1 0 00 11 = 0x1da3 (bad)

# CPLD: BRST=0 AA=1 PS=01(8b) EMAA=0 ASET=1(CS delay) AH=1(a&d hold)
# %00 0000 0 1 01 0 1 11 11 = 0x015f, 3d5f@WS=15
# %00 0000 0 1 01 0 0 00 11 = 0x0143
# %00 0000 0 1 01 0 0 11 11 = 0x014f

def setup_cs(cpu):
	cpu.CSAR0 = 0xf000,W
	cpu.CSMR0 = 0x001f,W
	cpu.CSCR0 = 0x09c3,W
	cpu.CSAR1 = 0x1000,W
	cpu.CSMR1 = 0x0000,W
	cpu.CSCR1 = 0x0143,W
	cpu.CSAR2 = 0x0000,W
	cpu.CSMR2 = 0x0000,W
	cpu.CSCR2 = 0x0000,W
	cpu.CSAR3 = 0x0000,W
	cpu.CSMR3 = 0x0000,W
	cpu.CSCR3 = 0x0000,W
	return


#
# Setup the DRAM controller.
#

# DRAM: 0x00000000 - 0x01ffffff (2^24-1, 16MB)

def setup_dram(cpu):
	# cpu.DCRR = 0x0027,W
	cpu.DCRR = 0x0000,W
	cpu.DCTR = 0x544b,W
	cpu.DCAR0 = 0x0000,W
	cpu.DCMR0 = 0x00fe0000
	# set *((long*)  $dcmr0) = 0x007E0000
	# set *((long*)  $dcmr0) = 0x013fffff
	# set *((char*)  $dccr0) = 0x13
	cpu.DCCR0 = (0x1f,B)
	# set *((char*)  $dccr0) = 0x93
	# set *((char*)  $dccr0) = 0xaf
	cpu.DCAR1 = 0x0000,W
	cpu.DCMR1 = 0x00000000
	cpu.DCCR1 = 0x00,B
	return

#
# Misc
#

def setup_misc(cpu):
	# set PAR5 (PST output)
	cpu.PAR = 0x20,W
	return

#
# Wake up the board
#

#define initBoard

def init(cpu):
	cpu.halt()
	setup_cs(cpu)
	setup_dram(cpu)
	setup_misc(cpu)
	return

def desc(cpu, l):
	for i in l:
		print i, hex(cpu.__getattr__(i))

MY_MBAR = 0xe0000000
cf = bdmcf5206(bdmpod(), MY_MBAR)
