#
# Init script for the Coldfire 5407 processor.
#

#	Copyright (C) 2003 Embedded CL S.A.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

MY_MBAR = 0xe0000000
MY_FLASH = 0xf0000000
MY_FLASH_BLOCKS = 64
MY_FLASH_SIZE = 128*1024*MY_FLASH_BLOCKS

from pybdm import *

# RAMBAR
# BA[31:13] xxxx WP 00 CI SC SD UC UD V

# MBAR (0xC0F movec)
# BA[31:10] 00000 SC SD UC UD V

#
#  Setup CSAR0 for the FLASH ROM.
#

# CSCR: 00 WS[3:0] 0 AA PS[1:0] BEM BSTR BSTW 000

def setup_cs(cpu):
# FLASH
	cpu.CSAR0 = 0xf000,W
	cpu.CSMR0 = ((MY_FLASH_SIZE-1) & 0xffff0000) | 0x0001
# 00 0111 0 1 11 0 0 0 000 # 28F320, 110ns (6ws)
#	cpu.CSCR0 = 0x1dc0,W
# 00 1000 0 1 11 0 0 0 000 # 28F640, 120ns (7ws)
	cpu.CSCR0 = 0x21c0,W

# CPLD
#	cpu.CSAR1 = 0x1000,W
#	cpu.CSMR1 = 0x1
# 00 0000 0 1 01 0 0 0 000
#	cpu.CSCR0 = 0x0140,W

# ETHERNET
	cpu.CSAR2 = 0x2000,W
	cpu.CSMR2 = 0x00000001
# 00 0000 0 0 11 1 0 0 000
	cpu.CSCR2 = 0x00e0,W

# USB
	cpu.CSAR3 = 0x3000,W
	cpu.CSMR3 = 0x00000001
	# 00 0000 0 1 11 0 0 0 000
	cpu.CSCR3 = 0x01c0,W
	return

#
# Setup the DRAM controller.
#

# DRAM: 0x00000000 - 0x03ffffff (2^26-1, 64MB)

# DCR(SO=1): SO 0 NAM COC IS RTIM[1:0] RC[8:0]
# DACR(SO=1): BA[13:0] 00 RE 0 CASL[1:0] 0 CBM[2:0] 0 IMRS PS[1:0] IP PM 00
# DMR: BAM[13:0] 000000000 WP 0 CI AM SC SD UC UD V

def setup_dram(cpu):
# 1 0 0 0 0 11 0x50
	cpu.DCR = 0x8650,W
# 14'b0 00 1 0 01 0 110 0 * 00 * 0 00 = 0x00009600
	cpu.DACR0 = 0x00009600
# 0x3fc
	cpu.DMR0 = 0x03fc0001
	cpu.DACR1 = 0x00
	cpu.DMR1 = 0x00
	return

#
# Misc
#

#def setup_misc(cpu):
# set PAR5 (PST output)
# set *(short*)$par = 0x20


#
# Wake up the board
#

def init(cpu):
	cpu.halt(.3)
	setup_cs(cpu)
	setup_dram(cpu)
#	setup_misc(cpu)
	return

#def desc(cpu, l):
#	k = l.keys()
#	k.sort()
#	for i in k:
#		print i, hex(cpu.__getattr__(i))
