"""pybdm

	Python BDM module

"""

#	Copyright (C) 2003 Embedded CL S.A.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from array import array
from bdmpod import * # POD interface class
from bdmcmd import * # BDM command layer
from bdmreg import * # BDM register abstraction
from bdmcpu import * # BDM CPU abstraction
from bdmflash import * # Flash utility classes

if __name__ == "__main__":
	# Some test routines...
	MY_PORT = 0x378 # parallel port base address
	p = bdmpod(port(MY_PORT)) # pod object that uses port object
	print 'Halting device...'
	p.halt()
	if not p.port.get_bit(*nHALT): print 'Halt bit on.'
	else: print 'Halt bit off.'

#	print 'Resetting device...',
#	b.reset()
#	if b.get_bit(nHALT): print 'Device is halted.'
#	else: print 'Device not halted'

#	print 'PST:', hex(b.pst())
#	for i in [0x2180, 0, 0]:
#		print hex(b.transfer(i)),

	# Test command layer
	c = bdmcmd(p) # command layer object
	cr = csr(c(RDMREG, CSR)) # read CSR into csr object
	print 'CSR = 0x%.8x' % int(cr)
	print 'CSR bits:'
	for i in cr.bit: print i, '\b:', eval('cr.'+i)
	# dump D & As
	for i in range(8): print 'D[%d] = 0x%.8x  A[%d] = 0x%.8x' % (i, c(RDREG,i), i, c(RAREG,i))

	MY_MBAR = 0xe0000000
	MY_RAMBAR = 0x20000000

	c(WCREG, MBAR, MY_MBAR+1) # set MBAR
	c(WCREG, RAMBAR0, MY_RAMBAR+1) # set RAMBAR

#
#	for i in range(0,1024*8):
#		c(WRITE, BYTE, MY_RAMBAR+i, i)

#	for i in range(0,1024*8):
#		a = MY_RAMBAR+i
#		print '0x%.2x = 0x%.2x' % (a, c(READ, BYTE, a))

#	for i in range(0,512/4):
#		a = MY_MBAR+i
#		print '0x%.8x = 0x%.8x' % (a, c(RDL, a))

	# dump 128 longs at current MBAR
	a = c(DPL, MY_MBAR, 128)
	print 'SIM register memory (128 longs):'
	for i in range(0,len(a),4):
		print '0x%.8x = 0x%.8x' % (i, a[i/4])

#	a = array('l')
#	f = file('ppbdm.h')
#	#f = file('test.bin')
#	try: a.fromfile(f, 8192)
#	except EOFError: pass
#	f.close()
#	# c(FILL, BYTE, MY_RAMBAR, a)
#	c(FLB, MY_RAMBAR, a)

	# fill SRAM memory with longs
	a = array('l')
	for i in range(1024*8/4): a.append(i)
	print 'Filling SRAM with increasing values...'
	c(FLL, MY_RAMBAR, a)

	# dump memory
	a = c(DUMP, LONG, MY_RAMBAR, 1024*8/4)
#	f = file('dump.bin', 'wb')
	print 'Dumping SRAM:'
	for i in range(0,len(a),4): print '0x%.8x' % i, '\b:', '0x%.8x' % a[i/4]
#	a.tofile(f)
#	f.close()

	creg = {
		'CACR': 0x002,
		'ACR0': 0x004,
		'ACR1': 0x005,
		'ACR2': 0x006,
		'ACR3': 0x007,
		'VBR': 0x801,
		'MACSR': 0x804,
		'MASK': 0x805,
		'ACC': 0x806,
		'SR': 0x80e,
		'PC': 0x80f,
		'RAMBAR': 0xc04,
		'RAMBAR0': 0xc04,
		'MBAR': 0xc0f
	}


	print 'Control registers:'
	k = creg.keys(); k.sort()
	for i in k:
		print '%s\t=0x%.8x' % (i, c(RCREG, creg[i]))
