# BDM CPU

#	Copyright (C) 2003 Embedded CL S.A.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from bdmpod import *
from bdmcmd import *


class bdmcf:
	Areg = {'A0':0, 'A1':1, 'A2':2, 'A3':3, 'A4':4, 'A5':5, 'A6':6, 'A7':7}
	Dreg = {'D0':0, 'D1':1, 'D2':2, 'D3':3, 'D4':4, 'D5':5, 'D6':6, 'D7':7}
	creg = {'MBAR': 0xc0f} # control registers
	dreg = { } # debug registers
	sreg = { } # sim registers (mbar offsets)

	def __init__(self, pod, mbar):
		self.pod = pod
		self.cmd = bdmcmd(pod)
		self.mbar = mbar & ~(0x1) #!FIXME: MCF5249 has two MBARs

	def __getattr__(self, k): # return self.mbar + self.__reg[k]
		if self.Areg.has_key(k): return self.cmd(RAREG, self.Areg[k])
		elif self.Dreg.has_key(k): return self.cmd(RDREG, self.Dreg[k])
		elif self.creg.has_key(k): return self.cmd(RCREG, self.creg[k])
		elif self.dreg.has_key(k): return self.cmd(RDMREG, self.dreg[k])
		elif self.sreg.has_key(k):
			kv = self.sreg[k]
			if type(kv) is tuple: return self.cmd(READ, kv[1], self.mbar+kv[0])
			else: return self.cmd(READ, LONG, self.mbar+kv)
		# return self.cmd(READ, self.sreg[k][0], self.sreg[k][1])
		elif type(k) is type(slice(0)): # [i:j]
			if k.stop == None: a = k.start
			else: a = k.stop
			return self.cmd(DUMP, LONG, k.start, (a-k.start)/4)
		else: return self.__dict__[k]

	def __setattr__(self, k, v):
		if self.Areg.has_key(k): self.cmd(WAREG, self.Areg[k], v)
		elif self.Dreg.has_key(k): self.cmd(WDREG, self.Dreg[k], v)
		elif self.creg.has_key(k): self.cmd(WCREG, self.creg[k], v)
		elif self.dreg.has_key(k): self.cmd(WDMREG, self.dreg[k], v)
		elif self.sreg.has_key(k):
			kv = self.sreg[k]
			if type(v) is tuple:
				if type(kv) is tuple: self.cmd(WRITE, v[1], self.mbar+kv[0], v[0])
				else: self.cmd(WRITE, v[1], self.mbar+kv, v[0])
			else:
				if type(kv) is tuple: self.cmd(WRITE, kv[1], self.mbar+kv[0], v)
				else: self.cmd(WRITE, LONG, self.mbar+kv, v)
		# return self.cmd(WRITE, self.sreg[k][0], self.sreg[k][1], v)
		else: self.__dict__[k] = v

	def __getitem__(self, k):
		if type(k) is tuple: return self.cmd(READ, k[1], k[0])
		elif type(k) is type(slice(0)):
			if k.stop == None: a = k.start
			else: a = k.stop
			if k.step == None: b = LONG
			else: b = k.step
			if b == BYTE: d = 1
			elif b == WORD: d = 2
			else: d = 4
			return self.cmd(DUMP, b, k.start, (a-k.start)/d)
		else: return self.cmd(READ, LONG, k)

	def __setitem__(self, k, v):
		import array
		if type(v) is tuple:
			if type(v[0]) is list or type(v[0]) is tuple: self.cmd(FILL, v[1], k, v)
			else: self.cmd(WRITE, v[1], k, v[0])
		elif type(v) is list or type(v) is tuple: self.cmd(FILL, LONG, k, v) # treat as array of long
		elif type(v) is array.ArrayType:
			if v.typecode == 'B' or v.typecode == 'b': self.cmd(FILL, BYTE, k, v)
			elif v.typecode == 'H' or v.typecode == 'h': self.cmd(FILL, WORD, k, v)
			else: self.cmd(FILL, LONG, k, v)
		else: self.cmd(WRITE, LONG, k, v)

	def halt(self, t=None):
		if t != None: self.pod.halt(t)
		else: self.pod.halt()
		self.cmd(WCREG, self.creg['MBAR'], self.mbar+1) # set valid bit


class bdmcf5206(bdmcf):
	creg = {
		'CACR': 0x002,
		'ACR0': 0x004,
		'ACR1': 0x005,
		'ACR2': 0x006,
		'ACR3': 0x007,
		'VBR': 0x801,
		'MACSR': 0x804,
		'MASK': 0x805,
		'ACC': 0x806,
		'SR': 0x80e,
		'PC': 0x80f,
		'RAMBAR': 0xc04,
		'RAMBAR0': 0xc04,
		'MBAR': 0xc0f
	}
	dreg = {
		'CSR': 0x00,
		'BAAR': 0x05,
		'AATR': 0x06,
		'TDR': 0x07,
		'PBR': 0x08,
		'PBMR': 0x09,
		'ABHR': 0x0c,
		'ABLR': 0x0d,
		'DBR': 0x0e,
		'DBMR': 0x0f,
		'AATR1': 0x16,
		'XTDR': 0x17,
		'PBR1': 0x18,
		'PBR2': 0x1a,
		'PBR3': 0x1b,
		'ABHR1': 0x1c,
		'ABLR1': 0x1d,
		'DBR1': 0x1e,
		'DBMR1': 0x1f
	}
	sreg = {
		'SIMR': 0x0003,
		'ICR1': 0x0014,
		'ICR2': 0x0015,
		'ICR3': 0x0016,
		'ICR4': 0x0017,
		'ICR5': 0x0018,
		'ICR6': 0x0019,
		'ICR7': 0x001a,
		'ICR8': 0x001b,
		'ICR9': 0x001c,
		'ICR10': 0x001d,
		'ICR11': 0x001e,
		'ICR12': 0x001f,
		'ICR13': 0x0020,
		'IMR': 0x0036,
		'IPR': 0x003a,
		'RSR': 0x0040,
		'SYPCR': 0x0041,
		'SWIVR': 0x0042,
		'SWSR': 0x0043,
		'DCRR': 0x0046,
		'DCTR': 0x004a,
		'DCAR0': 0x004c,
		'DCMR0': 0x0050,
		'DCCR0': 0x0057,
		'DCAR1': 0x0058,
		'DCMR1': 0x005c,
		'DCCR1': 0x0063,
		'CSAR0': 0x0064,
		'CSMR0': 0x0068,
		'CSCR0': 0x006e,
		'CSAR1': 0x0070,
		'CSMR1': 0x0074,
		'CSCR1': 0x007a,
		'CSAR2': 0x007c,
		'CSMR2': 0x0080,
		'CSCR2': 0x0086,
		'CSAR3': 0x0088,
		'CSMR3': 0x008c,
		'CSCR3': 0x0092,
		'CSAR4': 0x0094,
		'CSMR4': 0x0098,
		'CSCR4': 0x009e,
		'CSAR5': 0x00a0,
		'CSMR5': 0x00a4,
		'CSCR5': 0x00aa,
		'CSAR6': 0x00ac,
		'CSMR6': 0x00b0,
		'CSCR6': 0x00b6,
		'CSAR7': 0x00b8,
		'CSMR7': 0x00bc,
		'CSCR7': 0x00c2,
		'DMCR': 0x00c6,
		'PAR': 0x00ca,
		'TMR1': 0x0100,
		'TRR1': 0x0104,
		'TCR1': 0x0108,
		'TCN1': 0x010c,
		'TER1': 0x0111,
		'TMR2': 0x0120,
		'TRR2': 0x0124,
		'TCR2': 0x0128,
		'TCN2': 0x012c,
		'TER2 ': 0x0131
	}

#	def __init__(self, p, m):
#		bdmcf.__init__(self, p, m)


class bdmcf5407(bdmcf):
	creg = {
		'CACR': 0x002,
		'ACR0': 0x004,
		'ACR1': 0x005,
		'ACR2': 0x006,
		'ACR3': 0x007,
		'VBR': 0x801,
		'MACSR': 0x804,
		'MASK': 0x805,
		'ACC': 0x806,
		'SR': 0x80e,
		'PC': 0x80f,
		'RAMBAR0': 0xc04,
		'RAMBAR1': 0xc05,
		'MBAR': 0xc0f
	}
	dreg = {
		'CSR': 0x00,
		'BAAR': 0x05,
		'AATR': 0x06,
		'TDR': 0x07,
		'PBR': 0x08,
		'PBMR': 0x09,
		'ABHR': 0x0c,
		'ABLR': 0x0d,
		'DBR': 0x0e,
		'DBMR': 0x0f,
		'AATR1': 0x16,
		'XTDR': 0x17,
		'PBR1': 0x18,
		'PBR2': 0x1a,
		'PBR3': 0x1b,
		'ABHR1': 0x1c,
		'ABLR1': 0x1d,
		'DBR1': 0x1e,
		'DBMR1': 0x1f
	}
	sreg = {
		# SIM
		'RSR': 0x0000,
		'SYPCR': 0x0001,
		'SWIVR': 0x0002,
		'SWSR': 0x0003,
		'PAR': 0x0004,
		'IRQPAR': 0x0006,
		'PLLCR': 0x0008,
		'MPARK': 0x000c,
		'IPR': 0x0040,
		'IMR': 0x0044,
		'AVCR': 0x004b,
		'ICR0': 0x004c,
		'ICR1': 0x004d,
		'ICR2': 0x004e,
		'ICR3': 0x004f,
		'ICR4': 0x0050,
		'ICR5': 0x0051,
		'ICR6': 0x0052,
		'ICR7': 0x0053,
		'ICR8': 0x0054,
		'ICR9': 0x0055,
		'ICR10': 0x0056,
		'ICR11': 0x0057,
		# CS
		'CSAR0': 0x0080,
		'CSMR0': 0x0084,
		'CSCR0': 0x008a,
		'CSAR1': 0x008c,
		'CSMR1': 0x0090,
		'CSCR1': 0x0096,
		'CSAR2': 0x0098,
		'CSMR2': 0x009c,
		'CSCR2': 0x00a2,
		'CSAR3': 0x00a4,
		'CSMR3': 0x00a8,
		'CSCR3': 0x00ae,
		'CSAR4': 0x00b0,
		'CSMR4': 0x00b4,
		'CSCR4': 0x00ba,
		'CSAR5': 0x00bc,
		'CSMR5': 0x00c0,
		'CSCR5': 0x00c6,
		'CSAR6': 0x00c8,
		'CSMR6': 0x00cc,
		'CSCR6': 0x00d2,
		'CSAR7': 0x00d4,
		'CSMR7': 0x00d8,
		'CSCR7': 0x00de,
		# PP
		'PADDR': 0x0244,
		'PADAT': 0x0248,
		# DRAM
		'DCR': 0x0100,
		'DACR0': 0x0108,
		'DMR0': 0x010c,
		'DACR1': 0x0110,
		'DMR1': 0x0114,
		# UART
		'UMR0': 0x01c0,
		'USR0': 0x01c4,
		'UCSR0': 0x01c4,
		'UCR0': 0x01c8,
		'URB0': 0x01cc,
		'UTB0': 0x01cc,
		'UIPCR0': 0x01d0,
		'UACR0': 0x01d0,
		'UISR0': 0x01d4,
		'UIMR0': 0x01d4,
		'UBG10': 0x01d8,
		'UBG20': 0x01dc,
		'UIVR0': 0x01f0,
		'UIP0': 0x01f4,
		'UOP10': 0x01f8,
		'UOP00': 0x01fc,
		'UMR1': 0x0200,
		'RXLVL1': 0x0201,
		'MODCTL1': 0x0202,
		'TXLVL1': 0x0203,
		'USR1': 0x0204,
		'UCSR1': 0x0204,
		'RSMP1': 0x0206,
		'TSPC1': 0x0207,
		'UCR1': 0x0208,
		'URB1': 0x020c,
		'UTB1': 0x020c,
		'UIPCR1': 0x0210,
		'UACR1': 0x0210,
		'UISR1': 0x0214,
		'UIMR1': 0x0214,
		'UBG11': 0x0218,
		'UBG21': 0x021c,
		'UIVR1': 0x0230,
		'UIP1': 0x0234,
		'UOP11': 0x0238,
		'UOP01': 0x023c,
		# IC
		'MADR': 0x0280,
		'MFDR': 0x0284,
		'MBCR': 0x0288,
		'MBSR': 0x028c,
		'MBDR': 0x0290,
		# TIMER
		'TMR0': 0x0140,
		'TRR0': 0x0144,
		'TCR0': 0x0148,
		'TCN0': 0x014c,
		'TER0': 0x0151,
		'TMR1': 0x0180,
		'TRR1': 0x0184,
		'TCR1': 0x0188,
		'TCN1': 0x018c,
		'TER1': 0x0191,
		# DMA
		'SAR0': 0x0300,
		'DAR0': 0x0304,
		'DCR0': 0x0308,
		'BCR0': 0x030c,
		'DSR0': 0x0310,
		'DIVR0': 0x0314,
		'SAR1': 0x0340,
		'DAR1': 0x0344,
		'DCR1': 0x0348,
		'BCR1': 0x034c,
		'DSR1': 0x0350,
		'DIVR1': 0x0354,
		'SAR2': 0x0380,
		'DAR2': 0x0384,
		'DCR2': 0x0388,
		'BCR2': 0x038c,
		'DSR2': 0x0390,
		'DIVR2': 0x0394,
		'SAR3': 0x03c0,
		'DAR3': 0x03c4,
		'DCR3': 0x03c8,
		'BCR3': 0x03cc,
		'DSR3': 0x03d0,
		'DIVR3': 0x03d4
	}

#	def __init__(self, p, m):
#		bdmcf.__init__(self, p, m)
	