# BDM Flash

#	Copyright (C) 2003 Embedded CL S.A.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from pybdm import *

class strataflash:
	READ_ARRAY = 0xff
	READ_CODES = 0x90 # identifier codes
	READ_QUERY = 0x98
	READ_STATUS = 0x70
	CLEAR_STATUS = 0x50
	WRITE_BUFFER =  (0xe8, 0xd0)
	DATA_PROGRAM = 0x40 # 0x10
	BLOCK_ERASE = (0x20, 0xd0)
	PROGRAM_SUSPEND = 0xb0 # in block erase
	PROGRAM_RESUME = 0xd0
	CONFIGURATION = 0xb8
	SET_LOCK = (0x60, 0x01)
	CLEAR_LOCK = (0x60, 0xd0)
	PROTECT_PROGRAM = 0xc0

	class Error(Exception): pass

	class codes: pass
#	class interface: pass # system interface information
#	class geometry: pass # geometry information
	class query: pass

	def __init__(self, c, a):
		self.cpu, self.address = c, a
#		self. = None
		return

	def read_array(self):
		self.cpu[self.address] = (self.READ_ARRAY,W)
		return

	def read_status(self):
		self.cpu[self.address] = (self.READ_STATUS,W)
		r = self.cpu[self.address,W]
		self.cpu[self.address] = (self.READ_ARRAY,W)
		return r

	def read_codes(self):
		a, c = self.address, self.cpu
		c[a] = (self.READ_CODES, W)
		k = self.codes()
		k.manufacturer = c[a,W]
		k.device = c[a+2,W]
		# lock codes
		nb = (2**k.device) / (128*1024)
		k.lock = []
		for i in range(nb):
			k.lock.append(c[a+128*1024*i+4,W])
		c[a] = (self.READ_ARRAY,W)
		return k

#	def read_query(self):
#		a, c = self.address, self.cpu
#		c[a] = (self.READ_QUERY, W)
#		q = self.query()
#		q.manufacturer = c[a+(0x00<<1),W]
#		q.device = c[a+(0x01<<1),W]
#		# 'QRY'
#		q.qry = chr(c[a+(0x10<<1),W])
#		q.qry += chr(c[a+(0x11<<1),W])
#		q.qry += chr(c[a+(0x12<<1),W])
#		# System interface information
#		q.interface = c[a+(0x1b<<1),W]
#		# Device geometry information
#		q.geometry = c[a+(0x27<<1),W]
#
#		# block status
#		# intel table
#		return q

#	def read_query(self):
#		a, c = self.address, self.cpu
#		c[a] = (self.READ_QUERY, W)
#		r = c[a:a+(0x100<<1):W]
#		c[a] = (self.READ_ARRAY,W)
#		return r

	def read_query(self):
		a, c = self.address, self.cpu
		c[a] = (self.READ_QUERY, W)
		r = c[a:a+(0x100<<1):W]
		q = self.query()
		q.manufacturer = r[0] # 89h
		q.device = r[1] # 16h for 32Mb
		# CFI identification
		q.qry = chr(r[0x10]) + chr(r[0x11]) + chr(r[0x12])
		q.primary_id_code = r[0x14]<<8 | r[0x13] # 0100h
		q.primary_algorithm_address = r[0x16]<<8 | r[0x15] # 0x31
		q.alternate_id_code = r[0x18]<<8 | r[0x17] # 0000h
		q.secondary_algorithm_address = r[0x1a]<<8 | r[0x19] # 0000h
		# System interface information 0x1b-0x26
		# Device geometry information
		q.device_size = r[0x27] # 2^n bytes
		q.device_interface = r[0x29]<<8 | r[0x28] # 0002h for x8/x16
		q.write_buffer_size = r[0x2b]<<8 | r[0x2a] # 2^n in bytes
		q.erase_block_regions = r[0x2c] # 01h
		q.erase_block_region1 = r[0x30]<<24 | r[0x2f]<<16 | r[0x2e]<<8 | r[0x2d]
		# Primary-vendor specific extended query table
		p = q.primary_algorithm_address
		q.pri = chr(r[p+0]) + chr(r[p+1]) + chr(r[p+2])
		q.major_version = r[p+3]
		q.minor_version = r[p+4]
		f = r[p+8]<<24 | r[p+7]<<16 | r[p+6]<<8 | r[p+5]
		q.features = {
			'chip_erase': bool(f & 1<<0),
			'suspend_erase': bool(f & 1<<1),
			'suspend_program': bool(f & 1<<2),
			'legacy_lock_unlock': bool(f & 1<<3),
			'queued_erase': bool(f & 1<<4),
			'instant_block_locking': bool(f & 1<<5),
			'protection_bits': bool(f & 1<<6),
			'pagemode_read': bool(f & 1<<7),
			'synchronous_read': bool(f & 1<<8),
		}
		f = r[p+9]
		q.features['program_after_erase_suspend'] = bool(f & 1<<0)
		f = r[p+0xb]<<8 | r[p+0xa]
		q.features['block_lock_bit_sr_active'] = bool(f & 1<<0)
		q.features['block_lockdown_bit_sr_active'] = bool(f & 1<<1)
		q.features['vcc_highest_performance'] = r[p+0xc]
		q.features['vpp_optimum'] = r[p+0xd]
		q.features['protection_register_fields'] = r[p+0xe]
		f = r[p+0x12]<<24 | r[p+0x11]<<16 | r[p+0x10]<<8 | r[p+0x0f]
		q.features['protection_description'] = f
		q.features['page_mode_read'] = r[p+0x13]
		q.features['synchronous_mode_fields'] = r[p+0x13]

		# lock bits

		c[a] = (self.READ_ARRAY,W)
		return q


	def write_buffer(self, ofs, dat, to=1):
		"Write data to buffer. Data is treated as words." #! TODO: accept sequences, bytes and longs
		from time import time, sleep
		ld = len(dat)
		if not ld: return
		nwf = ld / 16 # 32-byte buffer
		nwl = ld % 16 # last write
		a, c = self.address, self.cpu
#		c[a] = (self.READ_ARRAY,W)
		i = 0 # wb counter i=0_nwf
		while i<nwf or (i==nwf and nwl):
			if i<nwf: nw = 16 # full page
			else: nw = nwl
			po = ofs + i*32 # ((i*16)<<1) # page offset
			# issue write command
			t = time()
			c[a+po] = (self.WRITE_BUFFER[0],W) ; xs = c[a+po,W]
			while not (xs & 0x80):
#				c[a] = (self.CLEAR_STATUS,W)
				c[a+po] = (self.WRITE_BUFFER[0],W) ; xs = c[a+po,W]
				if (time() - t) > to: raise self.Error('timeout')
			# write data to buffer and confirm
			c[a+po] = (nw-1,W)
			c[a+po] = dat[i*16:i*16+nw]
			c[a+po] = (self.WRITE_BUFFER[1],W)
			c[a+po] = (self.READ_STATUS,W)
			s = c[a+po,W]
			while not (s & 0x80):
				s = c[a+po,W]
				if (time() - t) > to: raise self.Error('timeout')
#			c[a] = (self.CLEAR_STATUS,W)
#			c[a] = (self.READ_ARRAY,W)
			i += 1
		c[a] = (self.READ_ARRAY,W)
		return


	def block_erase(self, ofs, to=0.1):
		from time import time
		a, c = self.address, self.cpu
		c[a] = (self.READ_ARRAY,W)
		t = time()
		c[a+ofs] = (self.BLOCK_ERASE[0],W)
		c[a+ofs] = (self.BLOCK_ERASE[1],W)
		c[a+ofs] = (self.READ_STATUS,W)
		s = c[a+ofs,W]
		while not (s & 0x80):
			s = c[a+ofs,W]
			if (time() - t) > to: raise self.Error('timeout')
		c[a] = (self.READ_ARRAY,W)
		return


	def block_lock(self, ofs, to=0.1):
		from time import time
		a, c = self.address, self.cpu
		c[a] = (self.READ_ARRAY,W)
		t = time()
		c[a+ofs] = (self.SET_LOCK[0],W)
		c[a+ofs] = (self.SET_LOCK[1],W)
		s = c[a+ofs,W]
		while not (s & 0x80):
			s = c[a+ofs,W]
			if (time() - t) > to: raise self.Error('timeout')
		if (s & (1<<3)): raise self.Error('voltage range error')
		if (s & (1<<5)) and (s & (1<<4)): raise self.Error('command sequence error')
		if (s & (1<<4)): raise self.Error('set lock-bit error')
		c[a] = (self.READ_ARRAY,W)
		return


	def block_unlock(self, ofs, to=0.1):
		from time import time
		a, c = self.address, self.cpu
		c[a] = (self.READ_ARRAY,W)
		t = time()
		c[a+ofs] = (self.CLEAR_LOCK[0],W)
		c[a+ofs] = (self.CLEAR_LOCK[1],W)
		s = c[a+ofs,W]
		while not (s & 0x80):
			s = c[a+ofs,W]
			if (time() - t) > to: raise self.Error('timeout')
		if (s & (1<<3)): raise self.Error('voltage range error')
		if (s & (1<<4)) and (s & (1<<5)): raise self.Error('command sequence error')
		if (s & (1<<5)): raise self.Error('set lock-bit error')
		c[a] = (self.READ_ARRAY,W)
		return


	def test_device_erase(self, to=0.1):
		a, c = self.address, self.cpu
		return

