# BDM pod classes

#	Copyright (C) 2003 Embedded CL S.A.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from ppbdm import *

DEFAULT_PORT_BASE = 0x378
DEFAULT_RESET_TIME = 0.1

# offsets
DATA_PORT = 0
STATUS_PORT = 1
CONTROL_PORT = 2

# DATA
D0 = (DATA_PORT, 0)
D1 = (DATA_PORT, 1)
D2 = (DATA_PORT, 2)
D3 = (DATA_PORT, 3)
D4 = (DATA_PORT, 4)
D5 = (DATA_PORT, 5)
D6 = (DATA_PORT, 6)
D7 = (DATA_PORT, 7)

# STATUS
nIRQ = (STATUS_PORT, 2)
nERROR = (STATUS_PORT, 3)
SLCT = (STATUS_PORT, 4)
PE = (STATUS_PORT, 5)
nACK = (STATUS_PORT, 6)
nBUSY = (STATUS_PORT, 7)

# CONTROL
nSTROBE = (CONTROL_PORT, 0)
nAUTOFEED = (CONTROL_PORT, 1)
nINIT = (CONTROL_PORT, 2)
nSLCTIN = (CONTROL_PORT, 3)
IRQEN = (CONTROL_PORT, 4)
DIRECTION = (CONTROL_PORT, 5)

# Port, Bit
DSI = D0
DSO = nBUSY # inverted
DSCLK = D1
nBKPT = D2
nRESET = D3
nTEA = D6
nHALT = nACK # inverted
PWR = SLCT
PST0 = nAUTOFEED
PST1 = nERROR
PST2 = nINIT
PST3 = nSLCTIN


# BDM status

STATUS_OK = 0x0ffff
NOT_READY = 0x10000
BUS_ERROR = 0x10001
ILLEGAL = 0x1ffff

class BdmError(Exception):
#	_desc = {
#		STATUS_OK: 'Status Ok',
#		NOT_READY: 'Not ready',
#		BUS_ERROR: 'Bus error',
#		ILLEGAL: 'Illegal operand'
#	}

	def __init__(self, *v):
		Exception.__init__(self, *v)
#		self.args = (self._desc[self[0]],) + self.args


class bdmpod:
	_input = {
		'DSO': DSO,
		'nHALT': nHALT,
		'PWR': PWR,
		'PST0': PST0,
		'PST1': PST1,
		'PST2': PST2,
		'PST3': PST3
	}
	_output = {
		'DSI': DSI,
		'DSCLK': DSCLK,
		'nBKPT': nBKPT,
		'nRESET': nRESET,
		'nTEA': nTEA
	}

	def __init__(self, p=None):
		if p: self.port = p
		else: self.port = port()

	def __getattr__(self, k):
		if self._input.has_key(k):
			return bool(self.port.get_bit(self._input[k][0], self._input[k][1]))
		elif self._output.has_key(k): # readback
			return bool(self.port.get_bit(self._output[k][0], self._output[k][1]))
		else: return self.__dict__[k]

	def __setattr__(self, k, v):
		if self._output.has_key(k):
			self.port.put_bit(self._output[k][0], self._output[k][1], v)
		else: self.__dict__[k] = v

	def __call__(self, v):
		r = self.port.transfer(v)
		if (r & 0x10000 and r != NOT_READY): raise BdmError(r, v)
#		if (r & 0x10000): raise BdmError(r)
		return r

	def inactive(self):
		self.nTEA = 1
		self.DSCLK = 0
		self.DSI = 1 #!

	def reset(self, t=DEFAULT_RESET_TIME):
		from time import sleep
		self.nRESET = 0
		self.nBKPT = 1
		sleep(t)
		self.nRESET = 1
		sleep(t)
		return

	def halt(self, t=DEFAULT_RESET_TIME):
		from time import sleep
		self.inactive()
		self.nBKPT = 1
		self.nRESET = 1
		sleep(t)
		self.nRESET = 0
		sleep(t) #!
		self.nBKPT = 0
		sleep(t)
		self.nRESET = 1
		sleep(t)
		self.nBKPT = 1
		return
