/*
	ppbdm  cross platform BDM python module
		Copyright 2003 embedded^CL <info@embedded.cl>
*/

/*
	Copyright (C) 2003 Embedded CL S.A.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ppbdm.h"

#ifdef MSW
static byte _giveio = 0;
#else
static byte _ioperm = 0;
#endif

/* Method table */
static PyMethodDef ppbdmport_methods[] = {
	{"port", (PyCFunction)ppbdmport_port, METH_VARARGS, "port base address"},
	{"get_byte", (PyCFunction)ppbdmport_get_byte, METH_VARARGS, "read byte from port"},
	{"put_byte", (PyCFunction)ppbdmport_put_byte, METH_VARARGS, "write byte to port"},
	{"get_bit", (PyCFunction)ppbdmport_get_bit, METH_VARARGS, "read bit from port"},
	{"put_bit", (PyCFunction)ppbdmport_put_bit, METH_VARARGS, "write bit to port"},
	{"transfer", (PyCFunction)ppbdmport_transfer, METH_VARARGS, "serial transfer"},

//	{"status", (PyCFunction)ppbdmport_status, METH_VARARGS},
	{NULL, NULL, 0, NULL}
};

#ifdef MSW
static byte
inportb(word adr) {
	byte result;

	__asm__ __volatile__(
		"movw %1, %%dx\n"
		"in %%dx, %%al\n"
		"movb %%al, %0"
		: "=g"(result)
		: "g"(adr)
		: "ax", "dx", "memory"
	);

	return result;
}

static void
outportb(word adr, byte val) {
	__asm__ __volatile__(
		"movw %0, %%dx\n"
		"movb %1, %%al\n"
//		"movw %1, %%ax\n"
		"out %%al, %%dx"
		:
		: "g"(adr), "g"(val)
		: "ax", "dx"
	);
}
#else
#define inportb(x) inb((x))
#define outportb(x,y) outb((y),(x))
#endif


static PyObject*
ppbdmport_alloc(word base) {
	Ppbdmport *po;

	po = PyObject_New(Ppbdmport, &Ppbdmport_Type);
	if (!po) return PyErr_NoMemory();

	po->port = base;

	return (PyObject*)po;
}

static void
ppbdmport_dealloc(Ppbdmport *po) {
	PyObject_Del(po);
}

//static PyObject*
//ppbdmport_reset(Ppbdmport *po, PyObject *ao) {
//
//	int t = 0;
//
//	if (!po) {
//		PyErr_BadInternalCall();
//		return NULL;
//	}
//
//	if (!PyArg_ParseTuple(ao, "|i", &t)) return NULL;
//
////	Sleep(1); // intergap
////	for (i=0; i<8; ++i) {
////		outportb(SK_P, SK_DN); // SO out
////
////		if (to_si & (1<<(7-i))) outportb(SI_P, 1<<SI_B); // write SI
////		else outportb(SI_P, 0<<SI_B);
////
////		if (inportb(SO_P) & (1<<SO_B)) fr_so |= 1<<(7-i); // read SO
////
////		outportb(SK_P, SK_UP); // SI in
////	}
//
//	Py_INCREF(Py_None); return Py_None;
//}

static PyObject*
ppbdmport_get_byte(Ppbdmport *po, PyObject *ao) {
	int b;

	if (!PyArg_ParseTuple(ao, "i:get_byte", &b)) {
		PyErr_BadArgument();
		return NULL;
	}

	b = inportb((word)(po->port + (b % 3)));

	return PyInt_FromLong(b);
}

static PyObject*
ppbdmport_put_byte(Ppbdmport *po, PyObject *ao) {
	int b, c;

	if (!PyArg_ParseTuple(ao, "ii:put_byte", &b, &c)) {
		PyErr_BadArgument();
		return NULL;
	}

	outportb((word)(po->port + (b % 3)), (byte)(c & 0xff));

	Py_INCREF(Py_None); return Py_None;
}

static PyObject*
ppbdmport_get_bit(Ppbdmport *po, PyObject *ao) {
	int b, c, t; // offset, bit

	if (!PyArg_ParseTuple(ao, "ii:get_bit", &b, &c)) {
		PyErr_BadArgument();
		return NULL;
	}

	b %= 3;
	c &= 0xff;
	t = inportb((word)(po->port + b));
	t = (t & 1<<c) ? 1 : 0;

	return PyInt_FromLong(t);
}

static PyObject*
ppbdmport_put_bit(Ppbdmport *po, PyObject *ao) {
	int b, c, d, t; // offset, bit

	if (!PyArg_ParseTuple(ao, "iii:put_bit", &b, &c, &d)) {
		PyErr_BadArgument();
		return NULL;
	}

	b %= 3;
	t = inportb((word)(po->port + b));
	if (d) t |= (1<<c) & 0xff;
	else t &= ~(1<<c) & 0xff;

	outportb((word)(po->port + b), (byte)t);

	Py_INCREF(Py_None); return Py_None;
}

static PyObject*
ppbdmport_transfer(Ppbdmport *po, PyObject *ao) {
	int i, m, t, r = 0;

	if (!PyArg_ParseTuple(ao, "i:transfer", &m)) {
		PyErr_BadArgument();
		return NULL;
	}

	// asume dsclk is down

	t = inportb((word)(po->port + DSI_P)); // also DSCLK_P
	for (i=0; i<17; ++i) {
		if (m & 1<<(16-i)) t |= 1<<DSI;
		else t &= ~(1<<DSI);
		outportb((word)(po->port + DSI_P), (byte) t & 0xff);
		t |= 1<<DSCLK;
		outportb((word)(po->port + DSCLK_P), (byte) t & 0xff);
//		Sleep(100);
		t &= ~(1<<DSCLK);
		outportb((word)(po->port + DSCLK_P), (byte) t & 0xff);
		r |= (inportb((word)(po->port + DSO_P)) & 1<<DSO) ? 0 : 1;
		r <<= 1;
	}

	return PyInt_FromLong(r>>1);
}

/*
// expand this to accept sequences
static PyObject*
ppbdmport_call_(Ppbdmport *po, PyObject *ao, PyObject *ko) // const char *pn, const uwioconf *xc) {
	int al;
	if (!po || !PyTuple_Check(ao)) {
		PyErr_BadInternalCall();
		return NULL;
	}

	al = PyTuple_Size(ao);

	if (al==0) {
		PyErr_BadArgument();
		return NULL;
	}
	else if (al==1) return ppbdmport_call_obj(po, PyTuple_GetItem(ao, 0));
	else return ppbdmport_call_obj(po, ao);
}

static PyObject*
ppbdmport_call_obj(Ppbdmport *po, PyObject *ao) {
	if (PyInt_Check(ao)) {
		po->i = (byte) (PyInt_AsLong(ao) & 0xff);
		po->o = (byte) (ppbdmport_sio(po, po->i) & 0xff);
		return PyInt_FromLong((long)po->o);
	}
	else if (PyString_Check(ao)) {
		int i;
		const int l = PyString_Size(ao);
		const char *s = PyString_AsString(ao);
		char r[l];

		for (i=0; i<l; ++i) r[i] = (char) (ppbdmport_sio(po, s[i]) & 0xff);
		po->i = (byte) s[l-1];
		po->o = (byte) r[l-1];
		return PyString_FromStringAndSize(r, l);
	}
	else return ppbdmport_call_seq(po, ao);
}

static PyObject*
ppbdmport_call_seq(Ppbdmport *po, PyObject *ao) {
	int i, l;
	PyObject *ro = NULL, *tmp;

	if (!PyTuple_Check(ao) && !PyList_Check(ao)) {
		PyErr_SetString(Uwioerror, "type not supported");
		return NULL;
	}

	l = PyObject_Length(ao);
	if (l == -1) return NULL;

	if (PyTuple_Check(ao)) {
		ro = PyTuple_New(l);
		if (!ro) return PyErr_NoMemory();

		for (i=0; i<l; ++i) {
			tmp = ppbdmport_call_obj(po, PyTuple_GetItem(ao, i));
			if (!tmp) {
				Py_DECREF(ro);
				return NULL;
			}
			PyTuple_SetItem(ro, i, tmp);
		}
	}
	else if (PyList_Check(ao)) {
		ro = PyList_New(l);
		if (!ro) return PyErr_NoMemory();

		for (i=0; i<l; ++i) {
			tmp = ppbdmport_call_obj(po, PyList_GetItem(ao, i));
			if (!tmp) {
				Py_DECREF(ro);
				return NULL;
			}
			PyList_SetItem(ro, i, tmp);
		}
	}

	return ro;
}
*/

static PyObject*
ppbdmport_port(Ppbdmport *po, PyObject *ao) {
	int np;
	if (!PyArg_ParseTuple(ao, "i:port", &np)) return NULL;

	po->port = (word) (np & 0xffff);

	Py_INCREF(Py_None); return Py_None;
}


//! todo: setattr for base
static PyObject*
ppbdmport_getattr(Ppbdmport *po, char *name) {
	if (!po) {
		PyErr_BadInternalCall();
		return NULL;
	}
	/*
	if (strcmp(name, "typecode") == 0) {
		char tc = a->ob_descr->typecode;
		return PyString_FromStringAndSize(&tc, 1);
	}
	if (strcmp(name, "itemsize") == 0) {
		return PyInt_FromLong((long)a->ob_descr->itemsize);
	}
	*/
/*
	if (strcmp(name, "base") == 0) {
		return PyInt_FromLong((long)po->port);
	}

	if (strcmp(name, "last") == 0) {
		return Py_BuildValue("(ii)", po->i, po->o);
	}
*/
	return Py_FindMethod(ppbdmport_methods, (PyObject*) po, name);
}

static PyObject*
new_ppbdmport(PyObject *so, PyObject *ao) {
	int b = DEFAULT_PORT;
#ifdef MSW
	HANDLE h;
#else
	byte r;
#endif

	if (!PyArg_ParseTuple(ao, "|i", &b)) return NULL;

#ifdef MSW
	// tell giveio we need access
	if (!_giveio) {
		h = CreateFile("\\\\.\\giveio", GENERIC_READ, 0, NULL,
			OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (h==INVALID_HANDLE_VALUE) {
			PyErr_SetString(Ppbdmerror, "giveio driver not found");
			return NULL;
		}
		CloseHandle(h);
		_giveio = 1;
	}
#else
	if (!_ioperm) {
		r = ioperm(b, 3, 1);
		if (r) {
			PyErr_SetString(Ppbdmerror, "giveio driver not found");
			return NULL;
		}
		_ioperm = 1;
	}
#endif

	return (PyObject*) ppbdmport_alloc(b);
}

static PyMethodDef ppbdm_methods[] = {
	{"port", (PyCFunction)new_ppbdmport, METH_VARARGS},
	{NULL, NULL}
};

statichere PyTypeObject Ppbdmport_Type = {
	PyObject_HEAD_INIT(NULL)
	0,
	"PortType",
	sizeof(Ppbdmport),
	0,
	(destructor)ppbdmport_dealloc,		/* tp_dealloc */
	0, /* (printfunc)array_print, */			/* tp_print */
	(getattrfunc)ppbdmport_getattr,		/* tp_getattr */
	0,					/* tp_setattr */
	0,					/* tp_compare */
	0, /* (reprfunc)array_repr,*/			/* tp_repr */
	0,					/* tp_as _number*/
	0, /* &ppbdmport_as_sequence,*/			/* tp_as _sequence*/
	0,					/* tp_as_mapping*/
	0, 					/* tp_hash */
	0, /* (ternaryfunc)ppbdmport_call_,	*/				/* tp_call */
	0,					/* tp_str */
	0,					/* tp_getattro */
	0,					/* tp_setattro */
	0, /* &array_as_buffer,*/			/* tp_as _buffer*/
	0, /* Py_TPFLAGS_DEFAULT, */			/* tp_flags */
	0, /* arraytype_doc,	*/			/* tp_doc */
 	0,					/* tp_traverse */
	0,					/* tp_clear */
	0,					/* tp_richcompare */
};

DL_EXPORT(void)
initppbdm() {
	PyObject *m, *d;

	Ppbdmport_Type.ob_type = &PyType_Type;
	m = Py_InitModule("ppbdm", ppbdm_methods);
	d = PyModule_GetDict(m);
	Ppbdmerror = PyErr_NewException("ppbdm.Error", PyExc_Exception, NULL);
	PyDict_SetItemString(d, "Error", Ppbdmerror);
	PyDict_SetItemString(d, "PortType", (PyObject*) &Ppbdmport_Type);
}
