#ifndef PPBDM_H
#define PPBDM_H

/*
	Copyright (C) 2003 Embedded CL S.A.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Python.h>

#ifdef STDC_HEADERS
#include <stddef.h>
#else
	#ifndef DONT_HAVE_SYS_TYPES_H
	#include <sys/types.h>
	#endif
#endif

#ifndef MSW
	#ifdef WIN32
	#define MSW
	#endif
#endif

#ifdef MSW
#include <windows.h>
#else
#include <unistd.h>
#include <fcntl.h>
#include <sys/io.h>
// #include <linux/ioctl.h>
#endif

#include "parport.h"

/* Macros */
#define _hi(x) (((x) >> 8) & 0x00ff)
#define _lo(x) ((x) & 0x00ff)

#define _cptr_to_word(x) ((*x) << 8 | *((x)+1))
#define _set(X,f) ((X) |= (f))
#define _rst(X,f) ((X) &= (~(f)))
#define _is_set(X,f) ((X) & (f))
#define _is_rst(X,f) (!((X) & (f)))

#define _get_byte(x,y) ((x>>(8*y)) & 0xff)
#define _get_word(x,y) ((x>>(16*y)) & 0xffff)
#define _get_dword(x,y) ((x>>(32*y)) & 0xffffffff)

#define _bytes_to_word(x,y) (((x)<<8) | (y))
#define _bytes_to_dword(w,x,y,z) (((w)<<24) | ((x)<<16) | ((y)<<8) | (z))
#define _words_to_dword(x,y) (((x)<<16) | (y))

/*
BDM
D0 = DSI
D1 = DSCLK
D2 = nBKPT
D3 = nRESET
D6 = nTEA
nAF = PST0
nErr = PST1
nIn = PST2
nSI = PST3
nAck = nHALT
nBsy = DSO
Sl = PWR
*/
#define DSI D0
#define DSI_P DATA_PORT
#define DSO nBUSY
#define DSO_P STATUS_PORT
#define DSCLK D1
#define DSCLK_P DATA_PORT
#define nBKPT D2
#define nBKPT_P DATA_PORT
#define nRESET D3
#define nRESET_P DATA_PORT
#define nTEA D6
#define nTEA_P DATA_PORT
#define nHALT nACK
#define nHALT_P STATUS_PORT
#define PWR SLCT
#define PWR_P STATUS_PORT
#define PST0 nAUTOFEED
#define PST0_P CONTROL_PORT
#define PST1 nERROR
#define PST1_P STATUS_PORT
#define PST2 nINIT
#define PST2_P CONTROL_PORT
#define PST3 nSLCTIN
#define PST3_P CONTROL_PORT

/* Types */
typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;
typedef signed char sbyte;
typedef signed short sword;
typedef signed long sdword;

static PyObject *Ppbdmerror;

// enum {};

#define Ppbdmport_Check(v)	((v)->ob_type == &Ppbdmport_Type)

typedef struct {
	PyObject_HEAD

	word port; // port base

#ifdef MSW
#else
#endif
} Ppbdmport;

staticforward PyTypeObject Ppbdmport_Type;

/* Declarations */
static PyObject* ppbdmport_alloc(word b);
static void ppbdmport_dealloc(Ppbdmport *xo);
// static int ppbdmport_sio(Ppbdmport *xo, int ib);

// static PyObject* ppbdmport_reset(Ppbdmport *po, PyObject *ao);

static PyObject* ppbdmport_getattr(Ppbdmport *po, char *name);
static PyObject* ppbdmport_port(Ppbdmport *po, PyObject *ao);
// static PyObject* ppbdmport_status(Ppbdmport *po, PyObject *ao);
// static PyObject* ppbdmport_call_(Ppbdmport *po, PyObject *ao, PyObject *ko);
// static PyObject* ppbdmport_call_obj(Ppbdmport *po, PyObject *ao);
// static PyObject* ppbdmport_call_seq(Ppbdmport *po, PyObject *ao);

static PyObject* ppbdmport_get_byte(Ppbdmport *po, PyObject *ao);
static PyObject* ppbdmport_put_byte(Ppbdmport *po, PyObject *ao);
static PyObject* ppbdmport_get_bit(Ppbdmport *po, PyObject *ao);
static PyObject* ppbdmport_put_bit(Ppbdmport *po, PyObject *ao);

static PyObject* ppbdmport_transfer(Ppbdmport *po, PyObject *ao);

#endif